@@ -4,20 +4,19 @@ import requests
 
 from alira.common.smtp import SMTP
 from alira.instance import Instance
-from alira.modules.module import InternalError
+from alira.modules.module import InternalError, ConnectionError
 from alira.modules.notification import Notification
 
 PIPELINE_EMAIL_MODULE_NAME = "email"
 
-
 class Email(Notification):
     def __init__(
         self,
         configuration_directory: str,
+        mapping_filename: str,
         sender: str,
         recipients: list,
         subject: str,
-        template_filename: str,
         filtering: str = None,
         files: list = None,
         files_directory: str = None,
@@ -28,12 +27,12 @@ class Email(Notification):
             configuration_directory=configuration_directory,
             module_id=PIPELINE_EMAIL_MODULE_NAME,
             filtering=filtering,
+            mapping_filename=mapping_filename,
             **kwargs,
         )
 
         self.sender = sender
         self.recipients = recipients
-        self.template_filename = template_filename
         self.subject = subject
         self.files = files
         self.files_directory = files_directory or "files"
@@ -62,19 +61,27 @@ class Email(Notification):
             self.relay = relay
 
     def run(self, instance: Instance, rest_authentication=None, **kwargs) -> dict:
-        super().run(instance, **kwargs)
+        try:
+            logging.info("Running email mapping function...")
+            super().run(instance, **kwargs)
+        except ConnectionError as e:
+            raise e
+        except Exception as e:
+            message = f"Error while executing email mapping function for pipeline {self.pipeline_id}"
+            logging.exception(message)
+            return {
+                "status": "FAILURE",
+                "message": message
+            }
 
         if self.filtering and not self.filtering(instance):
-            logging.info(
-                f"The instance didn't pass the filtering criteria. Instance: {instance}"
-            )
+            logging.info(f"The instance didn't pass the filtering criteria. Instance: {instance}")
             return {"status": "SKIPPED"}
 
         try:
-            self.template = self._load_template(instance, self.template_filename)
             response = self.relay.send(
                 instance=instance,
-                message=self.template,
+                message=self.html_text,
                 rest_authentication=rest_authentication,
                 **kwargs,
             )
@@ -84,12 +91,6 @@ class Email(Notification):
                     "status": "FAILURE",
                     "message": ("Failed to send email notification. " f"{response[1]}"),
                 }
-        except FileNotFoundError as e:
-            logging.exception(e)
-            return {
-                "status": "FAILURE",
-                "message": f"Template file {self.template_filename} not found",
-            }
         except Exception as e:
             logging.exception(e)
             return {
@@ -97,7 +98,10 @@ class Email(Notification):
                 "message": "There was an error sending the email notification",
             }
 
-        return {"status": "SUCCESS"}
+        return {
+            "status": "SUCCESS",
+            "attributes": self.attributes
+        }
 
 
 class SMTPRelay:
