@@ -25,8 +25,4 @@ class Map(Module):
         arguments = {**self.kwargs, **kwargs}
 
         result = fn(instance=instance, **arguments)
-
-        if not isinstance(result, dict):
-            raise RuntimeError("The result of the map operation must be a dictionary")
-
         return result
