@@ -1,11 +1,9 @@
 import logging
 import json
 import requests
-import base64
 import os
 
-from datetime import datetime
-
+from alira.common.database import Database
 from alira.instance import Instance
 from alira.modules.module import Connection, InternalError
 
@@ -29,7 +27,7 @@ class RestAuthentication:
                 response = self._request()
                 if response.status_code != 200:
                     raise InternalError(
-                        "There was an error authenticating with the Rest service. "
+                        f"There was an error authenticating with the Rest service. Status code: {response.status_code} "
                         f"Message: {response.text}"
                     )
                 token = json.loads(response.text)
@@ -65,6 +63,7 @@ class Rest(Connection):
         files: str = None,
         files_directory: str = None,
         upload_files: bool = True,
+        database_url: str = None,
         **kwargs,
     ):
         super().__init__(
@@ -77,47 +76,46 @@ class Rest(Connection):
         self.files = files
         self.files_directory = files_directory or "files"
         self.upload_files = upload_files
+        self.database_url = database_url
 
     def run(self, instance: Instance, rest_authentication, **kwargs):
         super().run(instance, **kwargs)
 
-        access_token = rest_authentication.get_access_token()
+        database = Database(self.database_url)
+        database.instance_dao().put(instance)
+        database.flat_instance_metadata_dao(instance.pipeline_id).put(instance)
 
-        url = "/".join(
-            map(
-                lambda x: str(x).rstrip("/"),
-                [rest_authentication.service, "instances", self.pipeline_id],
-            )
-        )
+        access_token = rest_authentication.get_access_token()
 
         try:
             files = self._get_files(instance)
-            response = self._request(instance, access_token, url, files)
+            response = self._request(rest_authentication, access_token, files)
 
             # If the access token we are currently using is expired, we want to
             # refresh it and try again.
             if response.status_code == 401:
                 access_token = rest_authentication.get_access_token(refresh=True)
-                response = self._request(instance, access_token, url, files)
+                response = self._request(rest_authentication, access_token, files)
 
             if response.status_code != 201:
                 raise InternalError(
-                    f"There was an error uploading instance. Message: {response.text}"
+                    f"Status Code: {response.status_code}. \
+                    There was an error uploading the files for instance {instance.instance_id}. \
+                    Message: {response.text}"
                 )
 
             result = json.loads(response.text)
 
             return {
-                "instance_id": result.get("instance_id", ""),
-                "files": result.get("files", []),
-                "timestamp": result.get("timestamp", ""),
+                "instance_id": instance.instance_id,
+                "files": result.get("filenames", [])
             }
 
         except InternalError as e:
             raise e
         except Exception as e:
             raise InternalError(
-                f"There was an error uploading instance. {e}"
+                f"There was an error uploading the files for instance {instance.instance_id}. {e}"
             ) from e
 
     def _get_files(self, instance: Instance):
@@ -137,34 +135,21 @@ class Rest(Connection):
                         self.configuration_directory, self.files_directory, filename
                     )
                     data = open(file_path, "rb").read()
-                    file = base64.b64encode(data).decode("utf-8")
                 except Exception:
                     logging.error(f"There was an error loading the file {file_path}")
-                    file = None
+                    data = None
             else:
-                file = None
+                data = None
 
-            result[filename] = file
+            result[filename] = data
 
         return result
 
-    def _request(self, instance: Instance, access_token: str, url: str, files: dict):
-        data = {
-            "instance_id": instance.instance_id,
-            "files": files,
-            "date": datetime.utcnow().isoformat(),
-            "metadata": instance.instance_metadata,
-            "properties": instance.instance_properties,
-        }
-
-        if instance.prediction is not None:
-            data["prediction"] = instance.prediction
-
-        if instance.confidence is not None:
-            data["confidence"] = instance.confidence
-
-        return requests.post(
-            url=url,
-            json=data,
-            headers={"Authorization": f"Bearer {access_token}"},
+    def _request(self, rest_authentication, access_token, files):
+        url = "/".join(
+            map(
+                lambda x: str(x).rstrip("/"),
+                [rest_authentication.service, "files", self.pipeline_id],
+            )
         )
+        return requests.post(url=url, files=files, headers={"Authorization": f"Bearer {access_token}"})
