@@ -16,13 +16,14 @@ from alira.common.database import Database
 
 rq_worker_logger = logging.getLogger("rq.worker")
 
-PIPELINE_FILENAME = "pipeline.yml"
+CONFIGURATION_FILENAME = "configuration.yml"
 PIPELINE_PYTHON_FILENAME = "pipeline.py"
 
 
 class Pipeline(object):
     def __init__(
         self,
+        pipeline_id: str = None,
         pipeline_configuration: any = None,
         configuration_directory: str = None,
         redis_server: str = None,
@@ -32,6 +33,7 @@ class Pipeline(object):
     ):
         self.kwargs = kwargs
 
+        self.pipeline_id = pipeline_id
         self.configuration_directory = configuration_directory or os.path.abspath(
             os.getcwd()
         )
@@ -39,12 +41,9 @@ class Pipeline(object):
         self.mysql_server = mysql_server
         self.asynchronous = asynchronous
 
-        # We are going to read the pipeline identifier and the list of
-        # modules from the pipeline.yml file.
-        self.pipeline_id = None
+        # We are going to read the list of modules from the pipeline.yml file.
         self.modules = None
         self.dependencies = None
-
         self._load_pipeline_configuration(pipeline_configuration)
 
     def run(self, data: dict) -> Instance:
@@ -93,12 +92,12 @@ class Pipeline(object):
                 **arguments
             )
 
-        instance_row = database.instance_dao().get_instance(instance_id=instance.instance_id)
-        return Instance.create_from_row(instance_row) if instance_row else instance
+        instance_from_database = Pipeline.get_instance_from_database(instance.instance_id, database=database)
+        return instance_from_database if instance_from_database else instance
 
     @classmethod
     def run_module(
-        self,
+        cls,
         module,
         module_signature,
         instance,
@@ -110,8 +109,8 @@ class Pipeline(object):
         rq_worker_logger.info(f"Running module {module_signature}...")
 
         Pipeline.load_pipeline_python_file(pipeline_id, configuration_directory)
-        instance_row = database.instance_dao().get_instance(instance_id=instance.instance_id)
-        instance = Instance.create_from_row(instance_row) if instance_row else instance
+        instance_from_database = Pipeline.get_instance_from_database(instance.instance_id, database=database)
+        instance = instance_from_database if instance_from_database else instance
 
         rq_worker_logger.debug(f"Input instance: {instance.to_dict()}")
         rq_worker_logger.debug(f"Dependencies: {dependencies}")
@@ -124,21 +123,33 @@ class Pipeline(object):
             result = module.run(instance, **dependencies)
 
             if result is None:
-                return None
+                result = dict()
 
-            module_output = {module_name: result}
-        except TypeError as e:
-            error = f"Unable to run module: {e}"
-            rq_worker_logger.exception(error)
-            module_output = {module_name: error}
+            if isinstance(result, dict):
+                if not "status" in result:
+                    result["status"] = "SUCCESS"
+            else:
+                raise RuntimeError(f"Result for module {module_name} must be a dictionary.")
         except InternalError as e:
             error = f"Internal Exception: {e}"
             rq_worker_logger.exception(error)
-            module_output = {module_name: error}
+            result = {
+                "status": "FAILURE",
+                "message": error
+            }
+        except Exception as e:
+            error = f"Unable to run module: {e}"
+            rq_worker_logger.exception(error)
+            result = {
+                "status": "FAILURE",
+                "message": error
+            }
+
+        module_output = {module_name: result}
 
         # Load the instance again to avoid overwriting another asynchronous module's data
-        instance_row = database.instance_dao().get_instance(instance_id=instance.instance_id)
-        instance = Instance.create_from_row(instance_row) if instance_row else instance
+        instance_from_database = Pipeline.get_instance_from_database(instance.instance_id, database=database)
+        instance = instance_from_database if instance_from_database else instance
 
         instance.instance_properties.update(module_output)
         instance.last_update_date = datetime.utcnow().isoformat()
@@ -148,7 +159,7 @@ class Pipeline(object):
         return module_output
 
     @classmethod
-    def load_pipeline_python_file(self, pipeline_id, configuration_directory):
+    def load_pipeline_python_file(cls, pipeline_id, configuration_directory):
         python_filename = os.path.join(
             configuration_directory, PIPELINE_PYTHON_FILENAME
         )
@@ -174,38 +185,65 @@ class Pipeline(object):
                     f"as module {module_name}"
                 )
 
+    @staticmethod
+    def get_instance_from_database(instance_id, database=None, mysql_server=None):
+        if database is None:
+            database = Database(mysql_server)
+        instance_row = database.instance_dao().get_instance(instance_id=instance_id)
+
+        if instance_row:
+            return Instance.create_from_row(instance_row)
+        return None
+
     def _load_pipeline_configuration(self, pipeline_configuration):
         if pipeline_configuration is None:
-            pipeline_configuration = os.path.join(
-                self.configuration_directory, PIPELINE_FILENAME
-            )
+            pipeline_configuration = os.path.join(self.configuration_directory, CONFIGURATION_FILENAME)
 
         if hasattr(pipeline_configuration, "read"):
-            pipeline_configuration = self._load_pipeline_configuration_from_stream(
-                pipeline_configuration
-            )
+            pipeline_configuration = self._load_pipeline_configuration_from_stream(pipeline_configuration)
         else:
-            pipeline_configuration = self._load_pipeline_configuration_from_file(
-                pipeline_configuration
-            )
+            pipeline_configuration = self._load_pipeline_configuration_from_file(pipeline_configuration)
 
-        self.pipeline_id = pipeline_configuration.get("name", None)
-        if self.pipeline_id is None:
-            raise ValueError(
-                "The name of the pipeline should be specified as part of "
-                "the configuration."
-            )
+        module_arguments = {}
+        for module_data in pipeline_configuration.get("default-module-configuration", []):
+            default_arguments = {
+                "pipeline_id": self.pipeline_id,
+                "configuration_directory": self.configuration_directory,
+            }
+            # Augment default_arguments with arguments from default-module-configuration and pipeline kwargs
+            for argument_id, argument in module_data.items():
+                default_arguments[argument_id] = argument
+            default_arguments.update(self.kwargs)
 
-        self.modules = list(
-            self._load_modules(pipeline_configuration, "pipeline").values()
-        )
-        self.dependencies = self._load_modules(pipeline_configuration, "dependencies")
+            # Store the module arguments in a dictionary with a unique reference
+            reference = self._get_reference(module_data, module_arguments.keys())
+            module_arguments[reference] = default_arguments
 
-    def _load_modules(self, pipeline_configuration, section: str):
+        self.dependencies = self._load_modules(pipeline_configuration, "dependencies", module_arguments)
+        self.modules = list(self._load_modules(pipeline_configuration, "pipeline", module_arguments).values())
+
+    def _load_modules(self, pipeline_configuration: dict, section: str, module_arguments: dict):
         result = {}
-        for section_data in pipeline_configuration.get(section, []):
+
+        modules_data = pipeline_configuration.get(section, {}).get(self.pipeline_id, [])
+
+        for module_data in modules_data:
+            overwrite_arguments = {}
+            for argument_id, argument in module_data.items():
+                overwrite_arguments[argument_id] = argument
+
+            reference = self._get_reference(module_data, result.keys())
+
+            final_arguments = module_arguments.get(reference, {})
+            final_arguments.update(overwrite_arguments)
+
             try:
-                module_path, _, module_name = section_data["module"].rpartition(".")
+                # Get module path/name from either `module_data` or `final_arguments` (allows specifying just module_id in the pipeline configuration)
+                module_str = module_data["module"] if "module" in module_data else final_arguments["module"]
+                module_path, _, module_name = module_str.rpartition(".")
+                del final_arguments["module"]  # Don't need to pass as a parameter to the module
+
+                # Import the module
                 module = getattr(import_module(module_path), module_name)
                 logging.info(f"Loaded module {module_path}.{module_name}")
             except ModuleNotFoundError as e:
@@ -213,33 +251,28 @@ class Pipeline(object):
                     f"Unable to load module {module_path}.{module_name}"
                 ) from e
 
-            arguments = {
-                "pipeline_id": self.pipeline_id,
-                "configuration_directory": self.configuration_directory,
-            }
-
-            for argument_id, argument in section_data.items():
-                if argument_id == "module":
-                    continue
-
-                arguments[argument_id] = argument
-
-            arguments.update(self.kwargs)
-
-            # Every module should have a `reference`. This will be the configured
-            # `module_id` or a snake_case name based on the module name.
-            if "module_id" in arguments:
-                reference = arguments["module_id"]
-            else:
-                reference = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", module_name)
-                reference = re.sub("([a-z0-9])([A-Z])", r"\1_\2", reference).lower()
-                while reference in result:
-                    reference += "_"
-
-            result[reference] = module(**arguments)
+            # Create the module instance and add it to the return result
+            result[reference] = module(**final_arguments)
 
         return result
 
+    def _get_reference(self, module_data: dict, keys: set):
+        # Every module should have a `reference`. This will be the configured
+        # `module_id` or a snake_case name based on the module name.
+
+        if "module_id" in module_data:
+            reference = module_data["module_id"]
+        else:
+            module_name = module_data["module"].rpartition(".")[2]
+            reference = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", module_name)
+            reference = re.sub("([a-z0-9])([A-Z])", r"\1_\2", reference).lower()
+
+        while reference in keys:
+            reference += "_"
+
+        return reference
+
+
     def _load_pipeline_configuration_from_stream(self, pipeline_configuration):
         try:
             return yaml.safe_load(pipeline_configuration.read())
